<?php

namespace Drp\Peek\Http\Controllers\Logs;

use Drp\Peek\Facade\Peek;
use Illuminate\Routing\Controller;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Str;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpFoundation\StreamedResponse;

class Show extends Controller
{
    /**
     * @param mixed $log
     * @return StreamedResponse|\Illuminate\Contracts\View\View
     */
    public function __invoke($log)
    {
        $accept = request()->headers->get('Accept');

        if (Str::contains($accept, 'text/html')) {
            $logContents = $this->getLogContents($log);

            return view('peek::logs.show', [
                'logContents' => $logContents,
                'logs' => Peek::logs(),
                'startBytes' => request('inverse') ? Storage::disk('peek:logs')->size($log) - 15000 : 15000,
                'bytesIncrement' => request('inverse') ? -45000 : 45000,
            ]);
        }

        $fileName = $filePath = $log;
        $stream = Storage::disk('peek:logs')->readStream($filePath);
        $fullSize = Storage::disk('peek:logs')->size($filePath);
        $responseCode = Response::HTTP_OK;

        $headers = [
            'Content-Disposition' => 'attachment; filename="' . $fileName . '"',
            'Content-Length' => $fullSize,
            'Content-Type' => Storage::disk('peek:logs')->mimeType($filePath),
            'Connection' => 'keep-alive',
        ];

        $range = request()->header('Range');
        if ($range === null) {
            abort(422);
        }

        [$unit, $byteRange] = explode('=', $range);
        [$start, $end] = explode('-', $byteRange);
        $success = fseek($stream, $start);
        if ($success === 0) {
            $size = $end - $start;
            $responseCode = Response::HTTP_PARTIAL_CONTENT;
            $headers['Content-Length'] = $size;
            $headers['Accept-Ranges'] = $unit;
            $headers['Content-Range'] = $unit . ' ' . $start . '-' . $end . '/' . $fullSize;
        }

        return new StreamedResponse(
            static function () use ($start, $end, $stream): void {
                $output = fopen('php://output', 'wb+');

                stream_copy_to_stream($stream, $output, $end - $start);
                fclose($stream);
                fclose($output);
            },
            $responseCode,
            $headers
        );
    }

    private function getLogContents(string $log): string
    {
        $maxSize = 15000;
        $size = Storage::disk('peek:logs')->size($log);

        if ($size > $maxSize) {
            $stream = Storage::disk('peek:logs')->readStream($log);

            return fread($stream, $maxSize);
        }

        return Storage::disk('peek:logs')->get($log);
    }
}
